@@ -4,7 +4,7 @@ use serde::{Deserialize, Serialize};
 use cosmwasm_std::{Binary, Decimal};
 use cw721::Expiration;
 
-use crate::{state::{CollectionInfo, Approval}, ContractError};
+use crate::{state::{CollectionInfo, Approval, TokenInfo, ModelInfo}, ContractError};
 
 #[derive(Serialize, Deserialize, Clone, Debug, PartialEq, JsonSchema)]
 pub struct InstantiateMsg {
@@ -42,6 +42,8 @@ pub enum ExecuteMsg<T> {
     CreateShoeModel(CreateShoeModelMsg<T>),
 
     Burn { token_id: String },
+
+    ModifyCollectionInfo {description: Option<String>, image: Option<String>, external_link: Option<String>, royalty_info: Option<RoyaltyInfoResponse> }
 }
 
 #[derive(Serialize, Deserialize, Clone, Debug, PartialEq, JsonSchema)]
@@ -117,11 +119,21 @@ pub enum QueryMsg {
         limit: Option<u32>,
     },
 
+    AllTokensInfo {
+        start_after: Option<String>,
+        limit: Option<u32>,
+    },
+
     AllModels {
         start_after: Option<String>,
         limit: Option<u32>,
     },
 
+    AllModelsInfo {
+        start_after: Option<String>,
+        limit: Option<u32>,
+    },
+
     Minter {},
 
     CollectionInfo {},
@@ -224,4 +236,14 @@ pub struct TokensResponse {
 #[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
 pub struct ModelsResponse {
     pub models: Vec<String>,
+}
+
+#[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
+pub struct AllNftsResponse<T> {
+    pub all_tokens_info: Vec<TokenInfo<T>>,
+}
+
+#[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
+pub struct AllModelsResponse<T> {
+    pub all_models_info: Vec<ModelInfo<T>>,
 }
\ No newline at end of file
